
*** begin macro call with parameters (default set for font size, top of the page and the max page, if missing) ***;
%macro pdf_annotate (sdtm_file,output_file,font_size=11,top_page=800,max_page=150);

PROC IMPORT DATAFILE = "&sdtm_file"
			OUT = _toc
			DBMS =XLSX REPLACE;
			GETNAMES = YES;
			SHEET='TOC';
RUN;

*** macrotize TOC data to know what tabs to import ***;
proc sql noprint;
select Dataset, Description into: domain1 -:domain999,:descrip1- :descrip999
from _toc;
%let numdom=&sqlobs;
quit;

*** deleting any work data with a _ prefix ***;
proc datasets lib=work nolist;
delete _:;
quit;


%do x = 1 %to &numdom;

*** import specific tabs from SDTM file ***;
	PROC IMPORT DATAFILE = "&sdtm_file"
				OUT = prep
				DBMS =EXCELCS REPLACE;
/*				GETNAMES = YES;*/
				SHEET="&&domain&x";
	RUN;

*** sort _prep data limiting to where origin contains CRF text, removing duplicates ***;
	proc sort data=prep (keep=Origin Variable_Name Define_Comment) nodupkey;
	by Variable_Name Origin Define_Comment;
	where index(upcase(origin),'CRF');
	run;

*** create specific domain datasets, creating needed variables ***;
	data _&&domain&x;
	set prep;length domain $10 new_define_comments variable $300 pages $100;
	domain="&&domain&x";
	pages=compress(origin,', ','kd');
	new_define_comments=define_comment;
	variable=Variable_Name;
	drop Origin Variable_Name Define_Comment;
	run;

*** end loop through each domain ***;
%end;

*** create format for domains based off of TOC data ***;
proc format;
value $descs
%do x = 1 %to &numdom;
"&&domain&x"="&&descrip&x"
%end;
;

*** create formats for background colors (see above for translation) ***;
value colors
1='0.75 1.0 1.0'
2='1.0 1.0 0.66'
3='1.0 0.75 0.66'
4='1.0 0.66 0.75'
5='0.66 0.75 1.0'
6='1.0 1.0 0.0'
7='0.0 0.75 1.0'
8='1.0 0.75 0.0'
9='0.75 1.0 0.75'
;

run;

*** set in all datasets (limit data to where domain ne missing) ***;
data _allin;
set _:;
if domain ne '';

*** create pages and num (the number of different pages mentioned in the document) ***;
pages=strip(compbl(tranwrd(pages,',',' ')));
num=countc(trim(pages),' ')+1;
run;

*** macrotize max number of pages ***;
proc sql noprint;
select max(num) into: maxpages
from _allin;
quit;

*** set in _allin data the number of time per &maxpages using a limiting statement to only select the data needed ***;
data _body;
set %do x =1 %to &maxpages;
_allin (in=x&x where=(num>=&x))
%end;
;

*** reset num to the set in occurance number ***;
%do x = 1 %to &maxpages;
if x&x then num=&x;
%end;

*** create the true page number per annotation ***;
page=input(scan(pages,num,' '),best.);

run;

*** sort body data removing duplicate annotations ***;
proc sort data=_body nodupkey;
by page domain variable;
run;

*** create header data per page and domain ***;
proc sort data=_body out=_headers (keep=domain page) nodupkey;
by page domain;
run;

*** bring in header and body data into one dataset ***;
data _whole;
set _headers (in=a) _body (in=b);
if a then type=1;
if b then type=2;

drop num pages;
run;

*** sort by page and type ***;
proc sort data=_whole;
by page type;
run;

*** create count of data per page and type ***;
data _whole_count;
set _whole;
count + 1;
by page type;
if first.type then count = 1;
run;

*** re-sort data for merging ***;
proc sort data=_whole_count;
by page domain;
run;

*** needed to ensure coloring of header and body annotation is the same ***;
*** merging the header data back onto the whole data and creating a new count variable ***;
*** this count variable becomes the coloring scheme ***;
data _prep; length text $300;
merge _whole_count (keep=domain page count type
where=(old_type=1) rename=(type=old_type count=ncount))
_whole_count;
by page domain;

*** modify the text presented in the annotation based on type ***;
*** fix DM variables to include DM. if missing ***;
if type=1 then text=strip(compbl(domain||' = '||put(domain,$descs.)));
else if type=2 then do;
	if upcase(domain)='DM' and strip(substr(upcase(variable),1,2)) not ='DM' then text='DM.'||strip(variable);
	else text=strip(variable);
end;

drop old_type;
run;

*** re-sort by page and type ***;
proc sort data=_prep;
by page type;
where page ne .;
run;

data _template;
do page = 2 to &max_page;
output;end;
run;

data _prep2;
merge _prep (in=a) _template (in=b);
by page;
if b and not a then do;
type=3;
text='For Annotations see PDF Page ';
count=1;
end;

run;

*** data to create the boxes for annotations ***;
data _rectangles;
set _prep2;length lrectx lrecty urectx urecty $5 textcolor $10 background $15 rect $50;
by page;

*** hold values of these variables ***;
retain h_x_adjust h_y_adjust b_x_adjust b_y_adjust;

*** adjust font size to be between 9-12  ***;
%if 8< &font_size >12 %then %let font_size=11;;
 
*** using the font size and length of text determine a constant value to create the length of the box ***;
font_adjust=&font_size-10;
txt_l=length(strip(text));
if txt_l <6 then constant=10+font_adjust;
else if txt_l <10 then constant=9+font_adjust;
else if txt_l <23 then constant=8+font_adjust;
else if txt_l <36 then constant=7+font_adjust;
else constant=6+font_adjust;

*** create boxes for header annotations ***;
if type=1 then do;

font_size=&font_size+3;

	if count=1 then do;
		h_x_adjust=0;
		h_y_adjust=&top_page;
	end;

*** if the box is about to reach the end of page then drop and move down ***;
	if h_x_adjust > 500 then do;
		h_x_adjust=0;
		h_y_adjust=h_y_adjust-23;
		lrectx= put(h_x_adjust+4,5.0);
		lrecty= put(h_y_adjust-20,5.0);
		urectx= put(h_x_adjust+(txt_l*constant),5.0);
		urecty= put(h_y_adjust,5.0);
		h_x_adjust=input(urectx,best.);
	end;
	else do;
		lrectx= put(h_x_adjust+4,5.0);
		lrecty= put(h_y_adjust-20,5.0);
		urectx= put(h_x_adjust+(txt_l*constant),5.0);
		urecty= put(h_y_adjust,5.0);
		h_x_adjust=input(urectx,best.);
	end;

*** create rect, textcolor and background color variables ***;
	rect=strip(compbl(lrectx||lrecty||urectx||urecty));
	textcolor='#000000';
	background=put(count,colors.);
end;

*** create boxes for body annotations ***;
if type=2 then do;
font_size=&font_size;
	if count=1 then do;
		b_y_adjust=h_y_adjust-19;
		b_x_adjust=0;
	end;

*** if the annotations reach the bottom of the page then moves over to the top of page ***;
	if b_y_adjust < 20 then do;
		b_x_adjust=b_x_adjust+120;
		b_y_adjust=h_y_adjust-19;
		lrectx= put(b_x_adjust+4,5.0);
		lrecty= put(b_y_adjust-16,5.0);
		urectx= put(b_x_adjust+(txt_l*constant),5.0);
		urecty= put(b_y_adjust,5.0);
		b_y_adjust=input(lrecty,best.)-2;
	end;
	else do;
		lrectx= put(b_x_adjust+4,5.0);
		lrecty= put(b_y_adjust-16,5.0);
		urectx= put(b_x_adjust+(txt_l*constant),5.0);
		urecty= put(b_y_adjust,5.0);
		b_y_adjust=input(lrecty,best.)-2;
	end;

*** create rect, textcolor and background color variables ***;
	rect=strip(compbl(lrectx||lrecty||urectx||urecty));
	textcolor='#FF0000';
	background=put(ncount,colors.);
end;
if type=3 then do;
font_size=&font_size+3;
		h_x_adjust=246;
		h_y_adjust=&top_page;
		lrectx= put(h_x_adjust+4,5.0);
		lrecty= put(h_y_adjust-16,5.0);
		urectx= put(h_x_adjust+(txt_l*constant+20),5.0);
		urecty= put(h_y_adjust,5.0);
	rect=strip(compbl(lrectx||lrecty||urectx||urecty));
	textcolor='#FF0000';
	background=put(count,colors.);
end;

run;

*** mactotize all data to use in the coding process ***;
proc sql noprint;
select count(*) into: max
from _rectangles;
%let max=&max;
select text, page, rect, textcolor, background, font_size into:
txt1 -: txt&max,: pg1-: pg&max,: rect1 -: rect&max,: txtclr1 -: txtclr&max,: bkgnd1 -: bkgnd&max,: fontsz1 -: fontsz&max
from _rectangles;
quit;

*** create the code ***;
data _coding;length code $1000;

*** begin code ***;
code='%FDF-1.2'; output;
code="1 0 obj<</FDF<</Annots["; output;

*** loop through number of annotations to be created ***;
%do x= 1 %to &max;
	code="%eval(&x+1) 0 R"; output;
%end;

code="]>>/Type/Catalog>>endobj"; output;

*** loop through number of annotations to be created ***;
%do x = 1 %to &max;
	code="%eval(&x+1) 0 obj<<"; output;
	code="/C[%trim(&&bkgnd&x)]"; output;
	code="/Contents(%str(&&txt&x))"; output;
	code="/DS(font: italic bold Arial,sans-serif &&fontsz&x...0pt; text-align:left; color:%trim(&&txtclr&x) )"; output;
	code="/F 4/Page %trim(%eval(&&pg&x-1))"; output;
	code="/Rect[%trim(&&rect&x)]"; output;
	code="/Subj(Text Box)/Subtype/FreeText/Type/Annot>>endobj"; output;
%end;

*** end code ***;
code="trailer"; output;
code="<</Root 1 0 R>>"; output;
code='%%EOF'; output;

run;

*** outputs the final code as an fdf document ***;
data _null_ ;
set _coding (keep=code where=(code ne ''));
FILE  "&output_file..fdf";
PUT code;
run ;

*** ends pdf_annotate macro ***;
%mend;

%pdf_annotate (SDTM_FILE =Z:\\Working\SDTM_FILE.xlsx, output_file = z:working\pdf annotations, FONT_SIZE = 12,TOP_PAGE=785,MAX_PAGE=4);
